/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sun Dec 16 17:21:40 EST 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation;  version 2 of the License, or           *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*   This program is more or less lifted from an example program
		provided by OD.  It has been slightly modified for this application
*/

/* Note that the type conversions from long to short for such values as
   numlayers, numblocks etc. limit the number of these entries that can
   be processed to 32768.  This should work for most applications; you can
   always use counters that are long ints instead of the short we have used
   to get the full range
*/


#define AD_PROTOTYPES

#define PATHCHAR '/'
#define DOT '.'

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <fcntl.h>
#include "ad2.h"
#include "odio.h"

#define AUDIT

void warningmessage(short errorno);
void myscanometer(short percent);
void myloadometer(short percent);
void mysaveometer(short percent);
void locprinthandle(AD_OBJHANDLE enthandle);
short constructinitfilepath (char *argv0,char *initfilepath);
short constructoutfilename (char *argv1,char *outfilename);
void setsaveversion(short argv4,short* saveversion, short* saver12dxfbvls);
void usage();

#ifdef AUDIT
short auditmsg(char *str)
{
  printf("%s\n",str);
/*  fgetc(stdin); */
  return(1);
}
#endif


short criterrhandler(short num)
{
  char ch;

/* file read errors can be retried */
  if (num==AD_CRITERR_FILEREADERROR) {
    printf("File read error: Abort/Retry?");
    ch=fgetc(stdin);
    if (ch=='R' || ch=='r') return(0);  /* return value ignored */
    exit(1);
  }

/* disk full can be retried */
  if (num==AD_CRITERR_DISKFULL) {
    printf("critical error: disk full\n");
/* Returning AD_DISKFULL_RETRY on a disk error means retry.  Returning
   AD_DISKFULL_CALLWRITEABORT means
   close the file and generate critical error AD_WRITE_ABORT (to allow
   a graceful exit).  Returning AD_DISKFULL_STOPWRITING means return
   errors back through the writing process to stop the file write. */
    return(AD_DISKFULL_STOPWRITING);
  }

/* write abort means disk full was reached and the program requested */
/* termination.  This is a fatal error. */
  if (num==AD_CRITERR_WRITEABORT) {
    printf("critical error: write abort\n");
/* You should delete the partial file here.  OD DOES NOT do so. */
    exit(1);
  }

/* malloc errors are fatal */
  if (num==AD_CRITERR_MALLOCERROR) {
    printf("critical error %d, aderror %d (%s), terminating\n",
      num,adError(),adErrorStr(adError()));
    exit(1);
  }

/* as are buffer overruns */
  if (num==AD_CRITERR_BUFFEROVERRUN) {
    printf("critical error -- buffer overrun, terminating\n");
    exit(1);
  }

  if (num==AD_CRITERR_BADDATA) {
    printf("critical error -- bad data detected, terminating\n");
    exit(1);
  }
  return(0);
}

int main(int argc,char **argv)
{
  AD_DB_HANDLE rdhandle;
	const short pathlen(256);
  short initerror;
  char initfilepath[pathlen];
	char outfilename[pathlen];
	short saveversion = 0; 						//	file save version
	short	saver12dxfbvls = 1;					//	for R11/R12 versions save R12 by default
	const short dxfprec = 8; 					//	decimal values for precision
	const short savedxfnegz  = 1;			//	save small neg numbers as -0.0 like ACAD
	const short savedxfwritezero = 1;	//	write z values as 0 like ACAD
  AD_OBJHANDLE temphandle;

//	if no arguments are supplied display some help information
  if (argc<2)
 {
	  printf("dwg2dxf -- Convert dwg files to dxf files\n");
  	printf("Released under the GNU Public License\n\n");
    printf("Usage: dwg2dxf infile [outfile] [version]\n\n");
		printf ("Required arguments:\n");
		printf ("infile:  is the path and name of the drawing file\n\n");
		printf ("Optional arguments:\n");
		printf ("outfile: is the path and name of the output dxf file.  If omitted the\n");
		printf ("\tpath and name of the input file will be used with the extension .dxf\n\n");
		printf ("version: is the AutoCAD version to be written.  If omitted the dxf file will\n");
		printf ("\tbe written in the same version as the drawing file.  Allowed values are:\n");
		printf ("\t\t25 for ACAD version 2.5\n");
		printf ("\t\t26 for ACAD version 2.6\n");
		printf ("\t\t9  for ACAD version 9\n");
		printf ("\t\t10 for ACAD version 10\n");
		printf ("\t\t11 for ACAD version 11\n");
		printf ("\t\t12 for ACAD version 12\n");
		printf ("\t\t13 for ACAD version 13\n");
		printf ("\t\t14 for ACAD version 14\n");
		printf ("\t\t2000 for ACAD version 2000\n");
    exit(1);
 }

//	try to find adinit.dat, if we can't display a message and exit
  if (!constructinitfilepath(argv[0],initfilepath))
 {
    printf("invalid argv[0] -- can't open init file\n");
    exit(1);
  }

//	define error handlers, try to initialize TookKit.  If we
//	can't display a message and exit
  adSetAd2CriticalErrorFn(criterrhandler);
  if (!adInitAd2(initfilepath,&initerror))
 {
    printf("Unable to initialize OpenDWG Toolkit\n");
    printf("error is %d: %s\n",initerror,adErrorStr(initerror));
    if (initerror==AD_UNABLE_TO_OPEN_INIT_FILE)
      printf("try moving adinit.dat to the current directory\n");
    exit(1);
  }

//	assign Toolkit callback functions, and setup for drawing reads
  adSetAd2WarningFn(warningmessage);
  adSetAd2LoadometerFn(myloadometer);
  adSetAd2SaveometerFn(mysaveometer);
  adSetAd2DxfscanometerFn(myscanometer);
  adSetupDwgRead();

//	try to read the drawing into memory, if we can't display a message and exit
  printf("Loading %s\n",argv[1]);
  if ((rdhandle=adLoadFile(argv[1],AD_PRELOAD_ALL,1))==0) {
    printf("error opening file %s: %s\n",argv[1],adErrorStr(adError()));
    if (adError()==AD_DUPLICATE_HANDLE_IN_FILE ||
        adError()==AD_DUPLICATE_HANDLE) {
      printf("handle: ");
      adErrorObjhandle(temphandle);
      locprinthandle(temphandle);
      printf("\n");
    }
    exit(1);
  }

//	perform an audit on the drawing
#ifdef AUDIT
  printf("Auditing %s\n",argv[1]);
  adAudit(rdhandle,AD_UNSAFEMODE, AD_VERBOSITY_QUIET, auditmsg);
  printf("Audit Complete\n");
#endif

//	if only one argument (and apparently a valid one) was provided
//	use it to create the file save name (same as read name except .dxf extension)
	if (argc < 3) constructoutfilename(argv[1],outfilename);
 	else strcpy(outfilename,argv[2]);


//	if a third argument is supplied, set the file save as version, otherwise
//	save version set the same as the file version read
	saveversion = adFileVersionLoaded(rdhandle);	
	if (argc > 3) setsaveversion(atoi(argv[3]),&saveversion,&saver12dxfbvls );
	
//	finally save the dxf file
	printf("Saving %s\n",outfilename);
  adSaveFile(
		rdhandle,
		outfilename,
		AD_DXF,
		saveversion,
		savedxfnegz,
		dxfprec,
		savedxfwritezero,
		saver12dxfbvls);

  if (adError()!=0) {
    printf("aderror is %d: %s\n",adError(),adErrorStr(adError()));
    if (adError()==AD_DUPLICATE_HANDLE_IN_FILE ||
        adError()==AD_DUPLICATE_HANDLE) {
      printf("handle: ");
      adErrorObjhandle(temphandle);
      locprinthandle(temphandle);
      printf("\n");
    }
    exit(1);
  }
  printf("File successfully created - closing dwg2dxf\n");
  adCloseFile(rdhandle);
  adCloseAd2();
}


short constructinitfilepath(char *argv0,char *initfilepath)
{
/* This function constructs a path to the init file, assuming that
   adinit.dat lies in the same directory as the program.  If this
   will not be true you should modify this function according to
   your own setup. */
  char *cptr;

  strcpy(initfilepath,argv0);
  if ((cptr=strrchr(initfilepath,PATHCHAR))==NULL) {
    strcpy(initfilepath,"adinit.dat");  /* current dir only */
    return(1);
  }
  strcpy(&cptr[1],"adinit.dat");
  return(1);
}

//	this function creates an output file name based upon the input file
//	name.  If the input file has no dot in the file name, a .dxf is appended
//	to the input name.  If there is a .anything in the file name than the
//	anything part is removed and dxf is substituted
	
short constructoutfilename(char *argv1,char *outfilename)
{
	char *cptr;

  strcpy(outfilename,argv1);
  if ((cptr=strrchr(outfilename,DOT))==NULL) {
    strcat(outfilename,".dxf");	//	if input file does not have a .anything extension
    return(1);
  }
  strcpy(&cptr[1],"dxf");					//	if input file does have a .anything extension
  return(1);
}

void warningmessage(short errorno)
{
  printf("OD Warning: %s\n",adErrorStr(errorno));
}


void myloadometer(short percent)
{
  printf("loading: %d\n",percent);
}

void mysaveometer(short percent)
{
  printf("saving: %d\n",percent);
}

void myscanometer(short percent)
{
  printf("scanning: %d\n",percent);
}

void locprinthandle(AD_OBJHANDLE enthandle)
{
  short i;
  char buf[30];
  char *cptr;

  cptr=buf;

  i=0;
  while (i<8 && enthandle[i]=='\0') i++;
  if (i<8) {
    sprintf(cptr,"%X",((unsigned)enthandle[i++]) & 255);  /* no leading 0 on first one */
    while (*cptr!=0) cptr++;
    while (i<8) {
      sprintf(cptr,"%.2X",((unsigned)enthandle[i++]) & 255);
      while (*cptr!=0) cptr++;
    }
  }
  else sprintf(cptr,"0");
  strcat(buf,"\n");
  printf(buf);
}

void setsaveversion(short d, short* saveversion, short* saver12dxfbvls)
{
	*saver12dxfbvls = 1;						//	R12 by default for R11/R12 files
	switch (d)
	{
    case 25:									
			*saveversion = AD_ACAD25;
			break;
		case 26:					
			*saveversion = AD_ACAD26;
			break;
		case 9:
			*saveversion = AD_ACAD9;
			break;
		case 10:
			*saveversion = AD_ACAD10;
			break;
		case 11:
			*saveversion = AD_ACAD11;		//	acad 11
			*saver12dxfbvls = 0;
			break;
		case 12:
			*saveversion = AD_ACAD11;    //	acad 12
			break;				
		case 13:
			*saveversion = AD_ACAD13;
			break;
		case 14:
			*saveversion = AD_ACAD14;
			break;
		case 2000:
			*saveversion = AD_ACAD2000;
			break;
	} 	//	file save version is already set to match the version of the drawing file upon entering
} 		//	this routine.  If no valid case above exists don't set anything, use the value as already set.


