/*************************************************************************************
odio.h as provided by OpenDWG with the following modifications:

	1)  removed all code for the #define OD_FAST_READ - this only applies to WinNT systems
	2)  added the #include to unistd.h which is where the function access resides
			it is in io.h for windows systems.
*****************************************************************************************/


#define OD_GENERIC_READ
#include <unistd.h>

#ifdef _cplusplus
  extern "C" {
#endif

/* odio functions */
/* write functions */
int odiow_fclose(void *f)
{
  return(fclose((FILE *)f));
}

void *odiow_fopen (void *path,char *accessmodes)
{
  FILE *tfile;

  if ((tfile=fopen((char*)path,accessmodes))==NULL) return(NULL);
/* setvbuf should increase performance somewhat */
#ifndef ODA_WINCE
  setvbuf(tfile,NULL,_IOFBF,8192);
#endif 
  return(tfile);
}

int odiow_fputs(const char *str,void *f)
{
  return(fputs(str,(FILE *)f));
}

int odiow_fread (void *buf, unsigned int size, unsigned int num, void *f)
{
  return(fread(buf,size,num,(FILE *)f));
}

int odiow_fseek (void *f, OdaLong offset, int whence)
{
  return(fseek((FILE *)f,offset,whence));
}

OdaLong odiow_ftell(void *f)
{
  return(ftell((FILE *)f));
}

int odiow_fwrite(void *b,int size,int count, void *f)
{
  return(fwrite(b,size,count,(FILE *)f));
}

#ifdef OD_GENERIC_READ
/* read functions */
int odior_fclose(void *f)
{
  return(fclose((FILE *)f));
}

short odior_feof(void *f)
{
  return(feof((FILE *)f));
}

unsigned char odior_fgetc(void *f)
{
  return(getc((FILE *)f));
}

#define OD_CR 13
#define OD_LF 10

char *odior_fgetdxfline(char *s, unsigned int n, void *f2)
{
  char lastchar=0;
  int onemorech;
  FILE *f=(FILE *)f2;
  char *origs=s;

  if (!n || feof(f)) return(NULL);
  while (n && lastchar!=OD_CR && lastchar!=OD_LF && !feof(f)) {
    *s++ = lastchar = getc(f);
    n--;
  }
  if (feof(f) || n==0) s--;
  *s=0;

/* we stopped on either \r or \n (or feof); we read one more
   character if it was \r and not eof to determine whether we had a cr/lf
   pair.  If we didn't, put the character back.  This fixes MAC text files
   which end in cr only */
  if (lastchar==OD_CR && !feof(f)) {
    onemorech=getc(f);
    if (!feof(f))  /* not really eof yet, next one is */
      if (onemorech!=OD_LF) ungetc(onemorech,f);
  }
/* added this to deal with the absurd case of lines terminated with \n\r */
  else if (lastchar==OD_LF && !feof(f)) {
    onemorech=getc(f);
    if (!feof(f))  /* not really eof yet, next one is */
      if (onemorech!=OD_CR) ungetc(onemorech,f);
  }
  return(origs);
}

OdaLong odior_flength(void *f2)   /* a dumb, but generic way to do this */
{                              /* depending on your platform, fstat may */
  FILE *f=(FILE *)f2;          /* be a better choice.  We have had some */
  OdaLong curloc,filelen;         /* trouble with it on Win95 networks, however */

  curloc=ftell(f);             /* hold location */
  fseek(f,0L,2);               /* seek to end */
  filelen=ftell(f);            /* get location, which is file size */
  fseek(f,curloc,0);           /* restore position */
  return(filelen);             /* return size */
}

void *odior_fopen (void *path)
{
  FILE *tfile;

  if ((tfile=fopen((char*)path,"rb"/*DNT*/))==NULL) return(NULL);
#ifndef ODA_WINCE
  setvbuf(tfile,NULL,_IOFBF,8192);
#endif //ODA_WINCE
  return(tfile);
}

int odior_fread (void *buf, unsigned int size, unsigned int num, void *f)
{
  return(fread(buf,size,num,(FILE *)f));
}

int odior_fseek (void *f, OdaLong offset, int whence)
{
  return(fseek((FILE *)f,offset,whence));
}

OdaLong odior_ftell(void *f)
{
  return(ftell((FILE *)f));
}
#endif


int odior_access(void *path, int mode)
{
	return access((const char *)path, mode);
}

#ifndef _CRTDBG_MAP_ALLOC
void *odmem_malloc(int size)
{
  return(malloc(size));
}

void odmem_free(void *loc)
{
  free(loc);
}
#endif /* _CRTDBG_MAP_ALLOC */

short odvm_init(void)
{
  return(1);
}

short odvm_term (void)
{
  return(1);
}

short odvm_readbytes(char *ptr,AD_VMADDR vmloc,unsigned short bytes)
{
  memcpy(ptr,vmloc,bytes);
  return(1);
}

short odvm_writebytes(AD_VMADDR vmloc,char *ptr,unsigned short bytes)
{
  memcpy(vmloc,ptr,bytes);
  return(1);
}

short odvm_free(AD_VMADDR ptr)
{
  free(ptr);
  return(1);
}

AD_VMADDR odvm_malloc(unsigned num_bytes)
{
  return((AD_VMADDR)malloc(num_bytes));
}

#ifdef _cplusplus
  }
#endif


